import * as eslexer from "es-module-lexer";
import glob from "fast-glob";
import fs from "fs";
import { bgGreen, bgMagenta, black, dim } from "kleur/colors";
import { fileURLToPath } from "url";
import * as vite from "vite";
import {
  createBuildInternals,
  eachPrerenderedPageData
} from "../../core/build/internal.js";
import { emptyDir, removeEmptyDirs } from "../../core/fs/index.js";
import { appendForwardSlash, prependForwardSlash } from "../../core/path.js";
import { isModeServerWithNoAdapter } from "../../core/util.js";
import { runHookBuildSetup } from "../../integrations/index.js";
import { PAGE_SCRIPT_ID } from "../../vite-plugin-scripts/index.js";
import { resolvedPagesVirtualModuleId } from "../app/index.js";
import { AstroError, AstroErrorData } from "../errors/index.js";
import { info } from "../logger/core.js";
import { getOutDirWithinCwd } from "./common.js";
import { generatePages } from "./generate.js";
import { trackPageData } from "./internal.js";
import { createPluginContainer } from "./plugin.js";
import { registerAllPlugins } from "./plugins/index.js";
import { getTimeStat } from "./util.js";
async function staticBuild(opts) {
  var _a, _b, _c;
  const { allPages, settings } = opts;
  if (isModeServerWithNoAdapter(opts.settings)) {
    throw new AstroError(AstroErrorData.NoAdapterInstalled);
  }
  const pageInput = /* @__PURE__ */ new Set();
  const facadeIdToPageDataMap = /* @__PURE__ */ new Map();
  const internals = createBuildInternals();
  const timer = {};
  timer.buildStart = performance.now();
  for (const [component, pageData] of Object.entries(allPages)) {
    const astroModuleURL = new URL("./" + component, settings.config.root);
    const astroModuleId = prependForwardSlash(component);
    trackPageData(internals, component, pageData, astroModuleId, astroModuleURL);
    pageInput.add(astroModuleId);
    facadeIdToPageDataMap.set(fileURLToPath(astroModuleURL), pageData);
  }
  if (((_c = (_b = (_a = settings.config) == null ? void 0 : _a.vite) == null ? void 0 : _b.build) == null ? void 0 : _c.emptyOutDir) !== false) {
    emptyDir(settings.config.outDir, new Set(".git"));
  }
  const container = createPluginContainer(opts, internals);
  registerAllPlugins(container);
  timer.ssr = performance.now();
  info(opts.logging, "build", `Building ${settings.config.output} entrypoints...`);
  const ssrOutput = await ssrBuild(opts, internals, pageInput, container);
  info(opts.logging, "build", dim(`Completed in ${getTimeStat(timer.ssr, performance.now())}.`));
  const rendererClientEntrypoints = settings.renderers.map((r) => r.clientEntrypoint).filter((a) => typeof a === "string");
  const clientInput = /* @__PURE__ */ new Set([
    ...internals.discoveredHydratedComponents,
    ...internals.discoveredClientOnlyComponents,
    ...rendererClientEntrypoints,
    ...internals.discoveredScripts
  ]);
  if (settings.scripts.some((script) => script.stage === "page")) {
    clientInput.add(PAGE_SCRIPT_ID);
  }
  timer.clientBuild = performance.now();
  const clientOutput = await clientBuild(opts, internals, clientInput, container);
  timer.generate = performance.now();
  await runPostBuildHooks(container, ssrOutput, clientOutput);
  switch (settings.config.output) {
    case "static": {
      await generatePages(opts, internals);
      await cleanServerOutput(opts);
      return;
    }
    case "server": {
      await generatePages(opts, internals);
      await cleanStaticOutput(opts, internals);
      info(opts.logging, null, `
${bgMagenta(black(" finalizing server assets "))}
`);
      await ssrMoveAssets(opts);
      return;
    }
  }
}
async function ssrBuild(opts, internals, input, container) {
  var _a, _b, _c;
  const { settings, viteConfig } = opts;
  const ssr = settings.config.output === "server";
  const out = ssr ? opts.buildConfig.server : getOutDirWithinCwd(settings.config.outDir);
  const { lastVitePlugins, vitePlugins } = container.runBeforeHook("ssr", input);
  const viteBuildConfig = {
    ...viteConfig,
    mode: viteConfig.mode || "production",
    logLevel: opts.viteConfig.logLevel ?? "error",
    build: {
      target: "esnext",
      ...viteConfig.build,
      emptyOutDir: false,
      manifest: false,
      outDir: fileURLToPath(out),
      copyPublicDir: !ssr,
      rollupOptions: {
        ...(_a = viteConfig.build) == null ? void 0 : _a.rollupOptions,
        input: [],
        output: {
          format: "esm",
          chunkFileNames: `chunks/[name].[hash].mjs`,
          assetFileNames: `${settings.config.build.assets}/[name].[hash][extname]`,
          ...(_c = (_b = viteConfig.build) == null ? void 0 : _b.rollupOptions) == null ? void 0 : _c.output,
          entryFileNames(chunkInfo) {
            if (chunkInfo.facadeModuleId === resolvedPagesVirtualModuleId) {
              return opts.buildConfig.serverEntry;
            } else {
              return "[name].mjs";
            }
          }
        }
      },
      ssr: true,
      minify: false,
      modulePreload: { polyfill: false },
      reportCompressedSize: false
    },
    plugins: [...vitePlugins, ...viteConfig.plugins || [], ...lastVitePlugins],
    envPrefix: viteConfig.envPrefix ?? "PUBLIC_",
    base: settings.config.base
  };
  await runHookBuildSetup({
    config: settings.config,
    pages: internals.pagesByComponent,
    vite: viteBuildConfig,
    target: "server",
    logging: opts.logging
  });
  return await vite.build(viteBuildConfig);
}
async function clientBuild(opts, internals, input, container) {
  var _a, _b, _c;
  const { settings, viteConfig } = opts;
  const timer = performance.now();
  const ssr = settings.config.output === "server";
  const out = ssr ? opts.buildConfig.client : getOutDirWithinCwd(settings.config.outDir);
  if (!input.size) {
    if (ssr) {
      await copyFiles(settings.config.publicDir, out, true);
    }
    return null;
  }
  const { lastVitePlugins, vitePlugins } = container.runBeforeHook("client", input);
  info(opts.logging, null, `
${bgGreen(black(" building client "))}`);
  const viteBuildConfig = {
    ...viteConfig,
    mode: viteConfig.mode || "production",
    logLevel: "info",
    build: {
      target: "esnext",
      ...viteConfig.build,
      emptyOutDir: false,
      outDir: fileURLToPath(out),
      rollupOptions: {
        ...(_a = viteConfig.build) == null ? void 0 : _a.rollupOptions,
        input: Array.from(input),
        output: {
          format: "esm",
          entryFileNames: `${settings.config.build.assets}/[name].[hash].js`,
          chunkFileNames: `${settings.config.build.assets}/[name].[hash].js`,
          assetFileNames: `${settings.config.build.assets}/[name].[hash][extname]`,
          ...(_c = (_b = viteConfig.build) == null ? void 0 : _b.rollupOptions) == null ? void 0 : _c.output
        },
        preserveEntrySignatures: "exports-only"
      }
    },
    plugins: [...vitePlugins, ...viteConfig.plugins || [], ...lastVitePlugins],
    envPrefix: viteConfig.envPrefix ?? "PUBLIC_",
    base: settings.config.base
  };
  await runHookBuildSetup({
    config: settings.config,
    pages: internals.pagesByComponent,
    vite: viteBuildConfig,
    target: "client",
    logging: opts.logging
  });
  const buildResult = await vite.build(viteBuildConfig);
  info(opts.logging, null, dim(`Completed in ${getTimeStat(timer, performance.now())}.
`));
  return buildResult;
}
async function runPostBuildHooks(container, ssrReturn, clientReturn) {
  const mutations = await container.runPostHook(ssrReturn, clientReturn);
  const config = container.options.settings.config;
  const buildConfig = container.options.settings.config.build;
  for (const [fileName, mutation] of mutations) {
    const root = config.output === "server" ? mutation.build === "server" ? buildConfig.server : buildConfig.client : config.outDir;
    const fileURL = new URL(fileName, root);
    await fs.promises.mkdir(new URL("./", fileURL), { recursive: true });
    await fs.promises.writeFile(fileURL, mutation.code, "utf-8");
  }
}
async function cleanStaticOutput(opts, internals) {
  const allStaticFiles = /* @__PURE__ */ new Set();
  for (const pageData of eachPrerenderedPageData(internals)) {
    allStaticFiles.add(internals.pageToBundleMap.get(pageData.moduleSpecifier));
  }
  const ssr = opts.settings.config.output === "server";
  const out = ssr ? opts.buildConfig.server : getOutDirWithinCwd(opts.settings.config.outDir);
  const files = await glob("**/*.mjs", {
    cwd: fileURLToPath(out)
  });
  if (files.length) {
    await eslexer.init;
    await Promise.all(
      files.map(async (filename) => {
        if (!allStaticFiles.has(filename)) {
          return;
        }
        const url = new URL(filename, out);
        const text = await fs.promises.readFile(url, { encoding: "utf8" });
        const [, exports] = eslexer.parse(text);
        let value = "const noop = () => {};";
        for (const e of exports) {
          value += `
export const ${e.n} = noop;`;
        }
        await fs.promises.writeFile(url, value, { encoding: "utf8" });
      })
    );
    removeEmptyDirs(out);
  }
}
async function cleanServerOutput(opts) {
  const out = getOutDirWithinCwd(opts.settings.config.outDir);
  const files = await glob("**/*.mjs", {
    cwd: fileURLToPath(out)
  });
  if (files.length) {
    await Promise.all(
      files.map(async (filename) => {
        const url = new URL(filename, out);
        await fs.promises.rm(url);
      })
    );
    removeEmptyDirs(out);
  }
  if (out.toString() !== opts.settings.config.outDir.toString()) {
    copyFiles(out, opts.settings.config.outDir);
    await fs.promises.rm(out, { recursive: true });
    return;
  }
}
async function copyFiles(fromFolder, toFolder, includeDotfiles = false) {
  const files = await glob("**/*", {
    cwd: fileURLToPath(fromFolder),
    dot: includeDotfiles
  });
  await Promise.all(
    files.map(async (filename) => {
      const from = new URL(filename, fromFolder);
      const to = new URL(filename, toFolder);
      const lastFolder = new URL("./", to);
      return fs.promises.mkdir(lastFolder, { recursive: true }).then(() => fs.promises.copyFile(from, to));
    })
  );
}
async function ssrMoveAssets(opts) {
  info(opts.logging, "build", "Rearranging server assets...");
  const serverRoot = opts.settings.config.output === "static" ? opts.buildConfig.client : opts.buildConfig.server;
  const clientRoot = opts.buildConfig.client;
  const assets = opts.settings.config.build.assets;
  const serverAssets = new URL(`./${assets}/`, appendForwardSlash(serverRoot.toString()));
  const clientAssets = new URL(`./${assets}/`, appendForwardSlash(clientRoot.toString()));
  const files = await glob(`**/*`, {
    cwd: fileURLToPath(serverAssets)
  });
  if (files.length > 0) {
    await fs.promises.mkdir(clientAssets, { recursive: true });
    await Promise.all(
      files.map(async (filename) => {
        const currentUrl = new URL(filename, appendForwardSlash(serverAssets.toString()));
        const clientUrl = new URL(filename, appendForwardSlash(clientAssets.toString()));
        return fs.promises.rename(currentUrl, clientUrl);
      })
    );
    removeEmptyDirs(serverAssets);
  }
}
export {
  staticBuild
};
